"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
function resolve(options) {
  const prefix = "\0plugin-resolve:";
  const resolveKeys = Object.keys(options);
  const resolveKeysWithPrefix = resolveKeys.map((key) => prefix + key);
  return [
    {
      name: "vite-plugin-resolve:resolveId",
      // Run before the builtin 'vite:resolve' of Vite
      enforce: "pre",
      resolveId(source) {
        if (resolveKeys.includes(source)) {
          return prefix + source;
        }
      }
    },
    {
      name: "vite-plugin-resolve",
      config(config) {
        if (!config.optimizeDeps)
          config.optimizeDeps = {};
        if (!config.optimizeDeps.exclude)
          config.optimizeDeps.exclude = [];
        let keys = resolveKeys;
        if (config.optimizeDeps.include) {
          keys = resolveKeys.filter(
            (key) => {
              var _a, _b;
              return !((_b = (_a = config.optimizeDeps) == null ? void 0 : _a.include) == null ? void 0 : _b.includes(key));
            }
          );
        }
        config.optimizeDeps.exclude.push(...keys);
      },
      load(id, opts) {
        if (resolveKeysWithPrefix.includes(id)) {
          const stringOrFunction = options[id.replace(prefix, "")];
          return typeof stringOrFunction === "function" ? stringOrFunction.apply(this, [id, opts]) : stringOrFunction;
        }
      }
    }
  ];
}
exports.default = resolve;
